@@ -1,6 +1,6 @@
 ;;;-*-Mode:LISP; Package: CHAOS; Base:10; Syntax:Common-lisp -*-
 ;;;
-;;; Copyright (c) 2000-2014, Toshimi Sawada. All rights reserved.
+;;; Copyright (c) 2000-2015, Toshimi Sawada. All rights reserved.
 ;;;
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -29,9 +29,9 @@
 ;;;
 (in-package :chaos)
 #|=============================================================================
-			     System:Chaos
-			    Module:BigPink
-			   File:types.lisp
+                             System:Chaos
+                            Module:BigPink
+                           File:types.lisp
 =============================================================================|#
 #-:chaos-debug
 (declaim (optimize (speed 3) (safety 0) #-GCL (debug 0)))
@@ -39,7 +39,7 @@
 (declaim (optimize (speed 1) (safety 3) #-GCL (debug 3)))
 
 ;;;                     *********************
-;;; 			BASIC Data Structures
+;;;                     BASIC Data Structures
 ;;;                     *********************
 
 #-(or draft-ansi-cl-2 ansi-cl :clisp)
@@ -62,7 +62,7 @@
 
 (defun list->queue (list)
   (cons list
-	(last list)))
+        (last list)))
 
 (defmacro queue-front-ptr (q)
   `(car ,q))
@@ -87,30 +87,30 @@
 (defun queue-insert (q item)
   (let ((new-pair (cons item nil)))
     (cond ((empty-queue? q)
-	   (queue-set-front-ptr q new-pair)
-	   (queue-set-rear-ptr q new-pair)
-	   q)
-	  (t
-	   (setf (cdr (queue-rear-ptr q)) new-pair)
-	   (queue-set-rear-ptr q new-pair)
-	   q))))
+           (queue-set-front-ptr q new-pair)
+           (queue-set-rear-ptr q new-pair)
+           q)
+          (t
+           (setf (cdr (queue-rear-ptr q)) new-pair)
+           (queue-set-rear-ptr q new-pair)
+           q))))
 
 (defun delete-queue (q)
   (cond ((empty-queue? q) nil)
-	(t (queue-set-front-ptr q
-				(cdr (queue-front-ptr q))))))
+        (t (queue-set-front-ptr q
+                                (cdr (queue-front-ptr q))))))
 
 ;;; =======
 ;;; LITERAL 
 ;;; =======
 (defstruct (literal (:print-function pr-literal) (:copier nil))
-  (clause nil :type (or null clause))	; containing clause
-  (atom nil :type (or null term))	; the body -- term
-  (sign t :type symbol)			; nil if negation
-  (type nil :type symbol)		; :pos-eq, :neg-eq, :evaluable
-					; :conditional-demod
-					; :normal-atom
-  (stat-bits 0 :type fixnum)		; various bit flags
+  (clause nil :type (or null clause))   ; containing clause
+  (atom nil :type (or null term))       ; the body -- term
+  (sign t :type symbol)                 ; nil if negation
+  (type nil :type symbol)               ; :pos-eq, :neg-eq, :evaluable
+                                        ; :conditional-demod
+                                        ; :normal-atom
+  (stat-bits 0 :type fixnum)            ; various bit flags
   )
 
 ;;; STAT-BITS
@@ -168,7 +168,7 @@
 (defmacro eq-literal? (literal)
   (once-only (literal)
    `(or (positive-eq-literal? ,literal)
-	(negative-eq-literal? ,literal))))
+        (negative-eq-literal? ,literal))))
 
 ;;; PROPOSITIONAL-LITERAL?
 ;;;
@@ -183,40 +183,40 @@
 #||
 (defun pr-literal (lit stream &rest ignore)
   (declare (type literal lit)
-	   (type stream stream)
-	   (ignore ignore))
+           (type stream stream)
+           (ignore ignore))
   (let ((.printed-vars-so-far. .printed-vars-so-far.))
     (unless (literal-sign lit)
       (princ "~(" stream)
       (setq .file-col. (1+ .file-col.)))
-    (with-in-module ((or *current-module* *last-module*))
+    (with-in-module ((get-context-module))
       (cond ((eq-literal? lit)
-	     (let* ((lhs (term-arg-1 (literal-atom lit)))
-		    (rhs (term-arg-2 (literal-atom lit)))
-		    (*print-indent* *print-indent*))
-	       (setq *print-indent* (max *print-indent*
-					  .file-col.))
-	       (setq .printed-vars-so-far.
-		 (append .printed-vars-so-far.
-			 (term-print lhs stream)))
-	       (setq .file-col. (file-column stream))
-	       (princ " = ")
-	       #||
-	       (if (print-check 0 30) ; 30?
-		   (princ "= ")
-		 (princ " = "))
-	       ||#
-	       (setq *print-indent*
-		 (max *print-indent*
-		      (setq .file-col. (file-column stream))))
-	       (setq .printed-vars-so-far.
-		 (append .printed-vars-so-far.
-			 (term-print rhs stream)))
-	       ))
-	    (t (setq .printed-vars-so-far.
-		 (append .printed-vars-so-far.
-			 (term-print (literal-atom lit) stream))))
-	    )
+             (let* ((lhs (term-arg-1 (literal-atom lit)))
+                    (rhs (term-arg-2 (literal-atom lit)))
+                    (*print-indent* *print-indent*))
+               (setq *print-indent* (max *print-indent*
+                                          .file-col.))
+               (setq .printed-vars-so-far.
+                 (append .printed-vars-so-far.
+                         (term-print lhs stream)))
+               (setq .file-col. (file-column stream))
+               (princ " = ")
+               #||
+               (if (print-check 0 30) ; 30?
+                   (princ "= ")
+                 (princ " = "))
+               ||#
+               (setq *print-indent*
+                 (max *print-indent*
+                      (setq .file-col. (file-column stream))))
+               (setq .printed-vars-so-far.
+                 (append .printed-vars-so-far.
+                         (term-print rhs stream)))
+               ))
+            (t (setq .printed-vars-so-far.
+                 (append .printed-vars-so-far.
+                         (term-print (literal-atom lit) stream))))
+            )
       )
     (unless (literal-sign lit)
       (princ ")" stream))
@@ -226,20 +226,19 @@
 
 (defun pr-literal (lit stream &rest ignore)
   (declare (type literal lit)
-	   (type stream stream)
-	   (ignore ignore))
+           (type stream stream)
+           (ignore ignore))
   (let ((.printed-vars-so-far. .printed-vars-so-far.))
     (unless (literal-sign lit)
       (princ "~(" stream)
       (setq .file-col. (1+ .file-col.)))
-    (with-in-module ((or *current-module* *last-module*))
+    (with-in-module ((get-context-module))
       (setq .printed-vars-so-far.
-	(append .printed-vars-so-far.
-		(term-print (literal-atom lit) stream))))
+        (append .printed-vars-so-far.
+                (term-print (literal-atom lit) stream))))
     (unless (literal-sign lit)
       (princ ")" stream))
-    .printed-vars-so-far.)
-  )
+    .printed-vars-so-far.))
 
 ;;; 
 ;;; some gobal flags
@@ -284,20 +283,20 @@
 ;;; ======
 
 (defstruct (clause (:print-function print-clause)
-	    ;; copier is defined in `clause.lisp'
-	    (:copier nil))
-  (parents nil :type list)		; parents produces this clause
-  (literals nil :type list)		; list of literal
+            ;; copier is defined in `clause.lisp'
+            (:copier nil))
+  (parents nil :type list)              ; parents produces this clause
+  (literals nil :type list)             ; list of literal
   (id -1 :type fixnum)
   (pick-weight -1 :type fixnum)
   (attributes nil :type list)
   (type nil :type symbol)
   (bits 0 :type fixnum)
   (heat-level 0 :type fixnum)
-  (formula nil :type (or null term))	; original formula
-  (axiom nil :type (or null axiom))	; derived axiom if any.
-  (container nil :type symbol)		; containing list, one of
-					; :sos, :usable, :other...
+  (formula nil :type (or null term))    ; original formula
+  (axiom nil :type (or null axiom))     ; derived axiom if any.
+  (container nil :type symbol)          ; containing list, one of
+                                        ; :sos, :usable, :other...
   )
 
 ;;; GET-CLAUSE : id -> Clause
@@ -337,13 +336,13 @@
 
 (defun print-clause (cl &optional (stream *standard-output*) &rest ignore)
   (declare (type clause cl)
-	   (type stream stream)
-	   (ignore ignore))
+           (type stream stream)
+           (ignore ignore))
   (let ((*print-pretty* nil)
-	(.printed-vars-so-far. nil)
-	(*print-xmode* :fancy)
-	(*standard-output* stream)
-	(fcol-1 0))
+        (.printed-vars-so-far. nil)
+        (*print-xmode* :fancy)
+        (*standard-output* stream)
+        (fcol-1 0))
     #||
     (when (symbolp cl)
       (format stream "~a" cl)
@@ -352,85 +351,85 @@
     (let ((flg nil))
       (declare (type symbol flg))
       (when (<= 0 (clause-id cl))
-	(format stream "~d:" (clause-id cl)))
+        (format stream "~d:" (clause-id cl)))
       (setq fcol-1 (file-column stream))
       ;; (when (< 0 (clause-heat-level cl))
       ;;    (format t "(heat=~D) " (clause-heat-level cl)))
       (princ "[" stream)
       (dolist (ips (clause-parents cl))
-	(declare (type list ips))
-	(dolist (ip ips)
-	  (declare (type (or symbol fixnum list) ip))
-	  (if (eq flg :colon)
-	      (princ ":" stream)
-	    (if (eq flg :comma)
-		(princ "," stream)))
-	  (cond ((symbolp ip)
-		 (setq flg :colon)
-		 (case ip
-		   (:binary-res-rule (princ "binary" stream))
-		   (:pbinary-res-rule (princ "prop-res" stream))
-		   (:hyper-res-rule (princ "hyper" stream))
-		   (:neg-hyper-res-rule (princ "neg-hyper" stream))
-		   (:ur-res-rule (princ "ur" stream))
-		   (:para-into-rule (princ "para-into" stream))
-		   (:para-from-rule (princ "para-from" stream))
-		   (:factor-rule (princ "factor" stream))
-		   ;; (:factor-simp-rule (princ "factor-simp" stream))
-		   (:factor-simp-rule (princ "fsimp" stream))
-		   (:distinct-constants (princ "dconst" stream))
-		   (:new-demod-rule (princ "new-demod" stream))
-		   (:back-demod-rule (princ "back-demod" stream))
-		   (:demod-rule (princ "demod" stream))
-		   (:unit-del-rule (princ "unit-del" stream))
-		   (:eval-rule (princ "eval" stream))
-		   (:copy-rule (princ "copy" stream))
-		   (:flip-eq-rule (princ "flip" stream))
-		   (:back-unit-del-rule (princ "back-unit-del" stream))
-		   (otherwise (princ ip stream))))
-		((atom ip)
-		 (setq flg :comma)
-		 (princ ip stream))
-		;; list
-		(t (setq flg :comma)
-		   (format stream "~a.~a" (car ip) (cdr ip)))))))
+        (declare (type list ips))
+        (dolist (ip ips)
+          (declare (type (or symbol fixnum list) ip))
+          (if (eq flg :colon)
+              (princ ":" stream)
+            (if (eq flg :comma)
+                (princ "," stream)))
+          (cond ((symbolp ip)
+                 (setq flg :colon)
+                 (case ip
+                   (:binary-res-rule (princ "binary" stream))
+                   (:pbinary-res-rule (princ "prop-res" stream))
+                   (:hyper-res-rule (princ "hyper" stream))
+                   (:neg-hyper-res-rule (princ "neg-hyper" stream))
+                   (:ur-res-rule (princ "ur" stream))
+                   (:para-into-rule (princ "para-into" stream))
+                   (:para-from-rule (princ "para-from" stream))
+                   (:factor-rule (princ "factor" stream))
+                   ;; (:factor-simp-rule (princ "factor-simp" stream))
+                   (:factor-simp-rule (princ "fsimp" stream))
+                   (:distinct-constants (princ "dconst" stream))
+                   (:new-demod-rule (princ "new-demod" stream))
+                   (:back-demod-rule (princ "back-demod" stream))
+                   (:demod-rule (princ "demod" stream))
+                   (:unit-del-rule (princ "unit-del" stream))
+                   (:eval-rule (princ "eval" stream))
+                   (:copy-rule (princ "copy" stream))
+                   (:flip-eq-rule (princ "flip" stream))
+                   (:back-unit-del-rule (princ "back-unit-del" stream))
+                   (otherwise (princ ip stream))))
+                ((atom ip)
+                 (setq flg :comma)
+                 (princ ip stream))
+                ;; list
+                (t (setq flg :comma)
+                   (format stream "~a.~a" (car ip) (cdr ip)))))))
     ;;
     (princ "] " stream)
     (let* ((.file-col. (file-column stream))
-	   (flg nil)
-	   (*print-indent* *print-indent*)
-	   (ind-check 0))
+           (flg nil)
+           (*print-indent* *print-indent*)
+           (ind-check 0))
       (declare (type symbol flg))
       (setq *print-indent* fcol-1)
       (if (print-check fcol-1 cl-print-mergine stream)
-	  (setq ind-check fcol-1)
-	(setq ind-check .file-col.))
+          (setq ind-check fcol-1)
+        (setq ind-check .file-col.))
       (setq *print-indent* ind-check)
       (dolist (lit (clause-literals cl))
-	(setq .file-col. (file-column stream))
-	(if flg
-	    (progn 
-	      (princ " | " stream)
-	      (setq .file-col. (+ 3 .file-col.))
-	      (if (print-check ind-check 20 stream)
-		  (setq .file-col. (file-column stream))
-		)
-	      )
-	  (setq flg t))
-	(setq .printed-vars-so-far.
-	  (append .printed-vars-so-far.
-		  (pr-literal lit stream))))
+        (setq .file-col. (file-column stream))
+        (if flg
+            (progn 
+              (princ " | " stream)
+              (setq .file-col. (+ 3 .file-col.))
+              (if (print-check ind-check 20 stream)
+                  (setq .file-col. (file-column stream))
+                )
+              )
+          (setq flg t))
+        (setq .printed-vars-so-far.
+          (append .printed-vars-so-far.
+                  (pr-literal lit stream))))
       )))
 
 #||
 (defun print-clause (cl &optional (stream *standard-output*) &rest ignore)
   (declare (type clause cl)
-	   (type stream stream)
-	   (ignore ignore))
+           (type stream stream)
+           (ignore ignore))
   (let ((*print-pretty* nil)
-	(.printed-vars-so-far. nil)
-	(*standard-output* stream)
-	(fcol-1 0))
+        (.printed-vars-so-far. nil)
+        (*standard-output* stream)
+        (fcol-1 0))
     (declare (special *print-pretty*))
     (let ((flg nil))
       (declare (type symbol flg))
@@ -440,44 +439,44 @@
       ;;    (format t "(heat=~D) " (clause-heat-level cl)))
       (princ "[" stream)
       (dolist (ips (clause-parents cl))
-	(declare (type list ips))
-	(dolist (ip ips)
-	  (declare (type (or symbol fixnum list) ip))
-	  (if (eq flg :colon)
-	      (princ ":" stream)
-	    (if (eq flg :comma)
-		(princ "," stream)))
-	  (cond ((symbolp ip)
-		 (setq flg :colon)
-		 (case ip
-		   (:binary-res-rule (princ "binary" stream))
-		   (:pbinary-res-rule (princ "prop-res" stream))
-		   (:hyper-res-rule (princ "hyper" stream))
-		   (:neg-hyper-res-rule (princ "neg-hyper" stream))
-		   (:ur-res-rule (princ "ur" stream))
-		   (:para-into-rule (princ "para-into" stream))
-		   (:para-from-rule (princ "para-from" stream))
-		   (:factor-rule (princ "factor" stream))
-		   ;; (:factor-simp-rule (princ "factor-simp" stream))
-		   (:factor-simp-rule (princ "fsimp" stream))
-		   (:distinct-constants (princ "dconst" stream))
-		   (:new-demod-rule (princ "new-demod" stream))
-		   (:back-demod-rule (princ "back-demod" stream))
-		   (:demod-rule (princ "demod" stream))
-		   (:unit-del-rule (princ "unit-del" stream))
-		   (:eval-rule (princ "eval" stream))
-		   (:copy-rule (princ "copy" stream))
-		   (:flip-eq-rule (princ "flip" stream))
-		   (:back-unit-del-rule (princ "back-unit-del" stream))
-		   (otherwise (princ ip stream))))
-		((atom ip)
-		 (setq flg :comma)
-		 (princ ip stream))
-		;; list
-		(t (setq flg :comma)
-		   (format stream "~a.~a" (car ip) (cdr ip)))
-		))
-	))
+        (declare (type list ips))
+        (dolist (ip ips)
+          (declare (type (or symbol fixnum list) ip))
+          (if (eq flg :colon)
+              (princ ":" stream)
+            (if (eq flg :comma)
+                (princ "," stream)))
+          (cond ((symbolp ip)
+                 (setq flg :colon)
+                 (case ip
+                   (:binary-res-rule (princ "binary" stream))
+                   (:pbinary-res-rule (princ "prop-res" stream))
+                   (:hyper-res-rule (princ "hyper" stream))
+                   (:neg-hyper-res-rule (princ "neg-hyper" stream))
+                   (:ur-res-rule (princ "ur" stream))
+                   (:para-into-rule (princ "para-into" stream))
+                   (:para-from-rule (princ "para-from" stream))
+                   (:factor-rule (princ "factor" stream))
+                   ;; (:factor-simp-rule (princ "factor-simp" stream))
+                   (:factor-simp-rule (princ "fsimp" stream))
+                   (:distinct-constants (princ "dconst" stream))
+                   (:new-demod-rule (princ "new-demod" stream))
+                   (:back-demod-rule (princ "back-demod" stream))
+                   (:demod-rule (princ "demod" stream))
+                   (:unit-del-rule (princ "unit-del" stream))
+                   (:eval-rule (princ "eval" stream))
+                   (:copy-rule (princ "copy" stream))
+                   (:flip-eq-rule (princ "flip" stream))
+                   (:back-unit-del-rule (princ "back-unit-del" stream))
+                   (otherwise (princ ip stream))))
+                ((atom ip)
+                 (setq flg :comma)
+                 (princ ip stream))
+                ;; list
+                (t (setq flg :comma)
+                   (format stream "~a.~a" (car ip) (cdr ip)))
+                ))
+        ))
     ;;
     (princ "] " stream)
     ))
@@ -485,7 +484,7 @@
 
 (defun pr-clause-list (cl &optional (detail nil))
   (declare (ignore detail)
-	   (type list cl))
+           (type list cl))
   (dolist (c cl)
     (print-next)
     (print-clause c)
@@ -500,80 +499,80 @@
 ;;;
 (defun literals-to-term (lit-list)
   (declare (type list lit-list)
-	   (values term))
+           (values term))
   (if (null lit-list)
       *bool-false*
     (let ((res nil))
       (declare (type (or null term) res))
       (do* ((lits lit-list (cdr lits))
-	    (l (car lits) (car lits)))
-	  ((null lits))
-	(declare (type literal l))
-	(if (literal-sign l)
-	    (push (make-term-with-sort-check
-		   *fopl-neg*
-		   (list (literal-atom l)))
-		  res)
-	  (push (literal-atom l) res)))
+            (l (car lits) (car lits)))
+          ((null lits))
+        (declare (type literal l))
+        (if (literal-sign l)
+            (push (make-term-with-sort-check
+                   *fopl-neg*
+                   (list (literal-atom l)))
+                  res)
+          (push (literal-atom l) res)))
       (if (cdr res)
-	  (setq res (make-right-assoc-normal-form-with-sort-check
-		     *fopl-or*
-		     (nreverse res)))
-	(setq res (car res)))
+          (setq res (make-right-assoc-normal-form-with-sort-check
+                     *fopl-or*
+                     (nreverse res)))
+        (setq res (car res)))
       ;;
       res)))
 
 (defun clause-to-term (cl)
   (declare (type clause cl)
-	   (values term))
+           (values term))
   (literals-to-term (clause-literals cl)))
 
 ;;; LITERAL COPIER
 ;;;
 (defun copy-literal (lit &optional variables clause subst)
   (declare (type literal lit)
-	   (type list variables subst)
-	   (type (or null clause) clause)
-	   (values literal))
+           (type list variables subst)
+           (type (or null clause) clause)
+           (values literal))
   (let ((atom (literal-atom lit)))
     (declare (type term atom))
     (when subst
       (setq atom (apply-subst subst atom)))
     (make-literal :clause (if clause
-			      clause
-			    (literal-clause lit))
-		  :atom (if variables
-			    (copy-term-using-variable atom
-						      variables)
-			  (copy-term-reusing-variables atom
-						      (term-variables atom)))
-		  :sign (literal-sign lit)
-		  :type (literal-type lit)))
+                              clause
+                            (literal-clause lit))
+                  :atom (if variables
+                            (copy-term-using-variable atom
+                                                      variables)
+                          (copy-term-reusing-variables atom
+                                                      (term-variables atom)))
+                  :sign (literal-sign lit)
+                  :type (literal-type lit)))
   )
 
 (defun shallow-copy-literal (lit &optional clause)
   (declare (type literal lit)
-	   (type (or null clause) clause)
-	   (values literal))
+           (type (or null clause) clause)
+           (values literal))
   (make-literal :clause (if clause
-			    clause
-			  (literal-clause lit))
-		:atom (literal-atom lit)
-		:sign (literal-sign lit)
-		:type (literal-type lit)))
+                            clause
+                          (literal-clause lit))
+                :atom (literal-atom lit)
+                :sign (literal-sign lit)
+                :type (literal-type lit)))
 
 
 ;;; CLAUSE-VARIABLES : Clause -> List[Variable]
 ;;;
 (defun clause-variables (clause)
   (declare (type clause clause)
-	   (values list))
+           (values list))
   (let ((vars nil))
     (declare (type list vars))
     (dolist (lit (clause-literals clause))
       (declare (type literal lit))
       (setq vars (nunion vars (term-variables (literal-atom lit))
-			 :test #'!term-eq)))
+                         :test #'!term-eq)))
     vars))
 
 ;;; CLAUSE-DISTINCT-VARIABLES (clause)
@@ -581,13 +580,13 @@
 
 (defun clause-distinct-variables (clause)
   (declare (type clause clause)
-	   (values fixnum))
+           (values fixnum))
   (length (clause-variables clause)))
 
 ;;; GROUND-CLAUSE? : Clause -> Bool
 ;;;
 (declaim (inline ground-clause?))
-	 
+         
 (defun ground-clause? (clause)
   (declare (type clause clause))
   (null (clause-variables clause)))
@@ -598,13 +597,13 @@
 
 (defun num-literals (clause)
   (declare (type clause clause)
-	   (values fixnum))
+           (values fixnum))
   (let ((num 0))
     (declare (type fixnum num))
     (dolist (lit (clause-literals clause))
       (declare (type literal lit))
       (unless (answer-literal? lit)
-	(incf num)))
+        (incf num)))
     num))
 
 ;;; NUM-ANSWERS : Clause -> Nat
@@ -613,13 +612,13 @@
 
 (defun num-answers (clause)
   (declare (type clause clause)
-	   (values fixnum))
+           (values fixnum))
   (let ((i 0))
     (declare (type fixnum i))
     (dolist (lit (clause-literals clause))
       (declare (type literal lit))
       (when (answer-literal? lit)
-	(incf i)))
+        (incf i)))
     i))
 
 ;;; NUM-LITERALS-ALL : Clause -> Nat
@@ -628,7 +627,7 @@
 
 (defun num-literals-all (clause)
   (declare (type clause clause)
-	   (values fixnum))
+           (values fixnum))
   (the fixnum (length (clause-literals clause))))
 
 ;;; UNIT-CLAUSE? : Clause -> Bool
@@ -649,10 +648,10 @@
 (defun positive-clause? (clause)
   (declare (type clause clause))
   (every #'(lambda (lit)
-	     (declare (type literal lit))
-	     (or (positive-literal? lit)
-		 (answer-literal? lit)))
-	 (clause-literals clause)))
+             (declare (type literal lit))
+             (or (positive-literal? lit)
+                 (answer-literal? lit)))
+         (clause-literals clause)))
 
 ;;; NEGATIVE-CLAUSE? : Clause -> Bool
 ;;;
@@ -661,10 +660,10 @@
 (defun negative-clause? (clause)
   (declare (type clause clause))
   (every #'(lambda (lit)
-	     (declare (type literal lit))
-	     (or (negative-literal? lit)
-		 (answer-literal? lit)))
-	 (clause-literals clause)))
+             (declare (type literal lit))
+             (or (negative-literal? lit)
+                 (answer-literal? lit)))
+         (clause-literals clause)))
 
 ;;; PROPOSITIONAL-CLAUSE? : Clause -> Bool
 ;;; 
@@ -673,11 +672,11 @@
 (defun propositional-clause? (clause)
   (declare (type clause clause))
   (every #'(lambda (lit)
-	     (declare (type literal lit))
-	     (let ((atom (literal-atom lit)))
-	       (and (not (term-is-variable? atom))
-		    (term-is-constant? atom))))
-	 (clause-literals clause)))
+             (declare (type literal lit))
+             (let ((atom (literal-atom lit)))
+               (and (not (term-is-variable? atom))
+                    (term-is-constant? atom))))
+         (clause-literals clause)))
 
 ;;; HORN-CLAUSE? : Clause -> Bool
 ;;; t if clause is a Horn Clause (at most one positive literal).
@@ -692,8 +691,8 @@
     (dolist (lit (clause-literals clause))
       (declare (type literal lit))
       (when (and (positive-literal? lit)
-		 (not (answer-literal? lit)))
-	(incf i)))
+                 (not (answer-literal? lit)))
+        (incf i)))
     (<= i 1)))
 
 ;;; EQUALITY-CLAUSE? : Clause -> Bool
@@ -706,8 +705,8 @@
   (dolist (lit (clause-literals clause))
     (declare (type literal lit))
     (if (or (positive-eq-literal? lit)
-	    (negative-eq-literal? lit))
-	(return-from equality-clause? t)))
+            (negative-eq-literal? lit))
+        (return-from equality-clause? t)))
   nil)
 
 ;;; SYMMETRY-CLAUSE? : Clause -> Bool
@@ -720,22 +719,22 @@
     (unless (= 2 (num-literals clause))
       (return-from symmetry-clause? nil))
     (let ((l1 (first lits))
-	  (l2 (second lits)))
+          (l2 (second lits)))
       (declare (type literal l1 l2))
       (when (eq (literal-sign l1)
-		(literal-sign l2))
-	(return-from symmetry-clause? nil))
+                (literal-sign l2))
+        (return-from symmetry-clause? nil))
       (and (eq-literal? l1)
-	   (eq-literal? l2)
-	   (let ((t1 (literal-atom l1))
-		 (t2 (literal-atom l2)))
-	     (and (term-is-variable? (term-arg-1 t1))
-		  (variable-eq (term-arg-1 t1)
-			       (term-arg-2 t2))
-		  (term-is-variable? (term-arg-2 t1))
-		  (variable-eq (term-arg-2 t1)
-			       (term-arg-1 t2))))))))
-	       
+           (eq-literal? l2)
+           (let ((t1 (literal-atom l1))
+                 (t2 (literal-atom l2)))
+             (and (term-is-variable? (term-arg-1 t1))
+                  (variable-eq (term-arg-1 t1)
+                               (term-arg-2 t2))
+                  (term-is-variable? (term-arg-2 t1))
+                  (variable-eq (term-arg-2 t1)
+                               (term-arg-1 t2))))))))
+               
 ;;; XX-RESOLVABLE : Clause -> Bool
 ;;; t if the non unit clause have a literal that can
 ;;; resolve with x = x.
@@ -746,14 +745,14 @@
     (declare (type literal lit))
     (when (negative-eq-literal? lit)
       (let* ((atom (literal-atom lit))
-	     (a1 (term-arg-1 atom))
-	     (a2 (term-arg-2 atom)))
-	(if (and (term-is-variable? a1)
-		 (not (occurs-in a1 a2)))
-	    (return-from xx-resolvable t)
-	  (if (and (term-is-variable? a2)
-		   (not (occurs-in a2 a1)))
-	      (return-from xx-resolvable t))))))
+             (a1 (term-arg-1 atom))
+             (a2 (term-arg-2 atom)))
+        (if (and (term-is-variable? a1)
+                 (not (occurs-in a1 a2)))
+            (return-from xx-resolvable t)
+          (if (and (term-is-variable? a2)
+                   (not (occurs-in a2 a1)))
+              (return-from xx-resolvable t))))))
   nil)
 
 
@@ -761,28 +760,28 @@
 ;;; proof system
 ;;; ============
 (defstruct psystem 
-  (module nil)				; context module
-  (sos nil)				; list of sos clause
-  (usable nil)				; list of usable clause
-  (passive nil)				; list of passive clause
-  (axioms nil)				; list of axioms in clause form
-  (demods nil)				; list of demod clauses
-  (bi-demods nil)			; list of builtin demod clauses
-  (clause-hash nil)			; hash table of clauses
-  (demodulators nil)			; (make-hash-table :test #'eq))
-					; demodulator hash table
-  (clause-counter 1)			; clause identifier counter
+  (module nil)                          ; context module
+  (sos nil)                             ; list of sos clause
+  (usable nil)                          ; list of usable clause
+  (passive nil)                         ; list of passive clause
+  (axioms nil)                          ; list of axioms in clause form
+  (demods nil)                          ; list of demod clauses
+  (bi-demods nil)                       ; list of builtin demod clauses
+  (clause-hash nil)                     ; hash table of clauses
+  (demodulators nil)                    ; (make-hash-table :test #'eq))
+                                        ; demodulator hash table
+  (clause-counter 1)                    ; clause identifier counter
   )
 
 (defun initialize-psystem (psys mod)
   (declare (type psystem psys)
-	   (type module mod)
-	   (values psystem))
+           (type module mod)
+           (values psystem))
   (setf (psystem-module psys) mod
-	(psystem-sos psys) nil
-	(psystem-usable psys) nil
-	(psystem-axioms psys) nil
-	(psystem-clause-counter psys) 1)
+        (psystem-sos psys) nil
+        (psystem-usable psys) nil
+        (psystem-axioms psys) nil
+        (psystem-clause-counter psys) 1)
   (clrhash (psystem-clause-hash psys))
   (clrhash (psystem-demodulators psys))
   psys)
@@ -792,50 +791,50 @@
 ;;; allocated one for each clashable literal of nucleus.
 ;;;
 (defstruct (clash (:print-function print-clash))
-  (literal nil :type (or null literal))	; literal from nucleus
-  (db nil :type (or null hash-table))	; indexed table to use for
-					;  finding satellites
-  (subst nil :type list)		; unifying substitution
+  (literal nil :type (or null literal)) ; literal from nucleus
+  (db nil :type (or null hash-table))   ; indexed table to use for
+                                        ;  finding satellites
+  (subst nil :type list)                ; unifying substitution
   (clashables nil :type list)
   (found-lit nil :type (or null literal)) ; unifying literal
-  (evaluable nil)			; bi-demod
-  (evaluation nil)			; ditto
-  (already-evaluated nil)		; ditto
-  (prev nil :type (or null clash))	; links
+  (evaluable nil)                       ; bi-demod
+  (evaluation nil)                      ; ditto
+  (already-evaluated nil)               ; ditto
+  (prev nil :type (or null clash))      ; links
   (next nil :type (or null clash))
   )
 
 (defun print-clash (obj &optional (stream *standard-output*)
-		    &rest ignore)
+                    &rest ignore)
   (declare (ignore ignore))
   (let* ((*standard-output* stream)
-	 (fcol (file-column stream))
-	 (*print-indent* (if (not (= 0 fcol))
-			     fcol
-			   (+ *print-indent* 4))))
+         (fcol (file-column stream))
+         (*print-indent* (if (not (= 0 fcol))
+                             fcol
+                           (+ *print-indent* 4))))
     ;;
     (declare (type fixnum fcol *print-indent*))
     (do ((clash obj (clash-next clash))
-	 (num 0 (1+ num)))
-	((null clash))
+         (num 0 (1+ num)))
+        ((null clash))
       (declare (type fixnum num))
       (format t "#~d<clash: lit = " num)
       (prin1 (clash-literal clash))
       (print-next)
       (format t "clause-id = ~d" (clause-id (literal-clause
-					     (clash-literal clash))))
+                                             (clash-literal clash))))
       (print-next)
       (princ "subst = ") (print-substitution (clash-subst clash))
       (print-next)
       (princ "found-lit = ") (prin1 (clash-found-lit clash))
       (print-next)
       (when (clash-found-lit clash)
-	(format t "found clause-id = ~d"
-		(clause-id (literal-clause (clash-found-lit clash)))))
+        (format t "found clause-id = ~d"
+                (clause-id (literal-clause (clash-found-lit clash)))))
       (when (clash-evaluable clash)
-	(format t "evaluable: value = ~a" (clash-evaluation clash))
-	(print-next)
-	(format t "already evaled? = ~a" (clash-already-evaluated clash)))
+        (format t "evaluable: value = ~a" (clash-evaluation clash))
+        (print-next)
+        (format t "already evaled? = ~a" (clash-already-evaluated clash)))
       (princ ">")
       (print-next)
     )))
@@ -862,9 +861,9 @@
 ;;; DEMODULATOR
 ;;;
 (defstruct (demod
-	    (:copier nil)
-	    (:constructor make-demod)
-	    (:print-function print-demodulator))
+            (:copier nil)
+            (:constructor make-demod)
+            (:print-function print-demodulator))
   (axiom nil :type (or null axiom))
   (order :normal :type symbol)
   (clause nil)
@@ -887,26 +886,26 @@
 (defun print-demod-object (obj &optional (stream *standard-output*) &rest ignore)
   (declare (ignore ignore))
   (format stream "#<demodulator (~a) ~a: ~x>"
-	  (clause-id (demod-clause obj))
-	  (demod-order obj) (addr-of obj)))
+          (clause-id (demod-clause obj))
+          (demod-order obj) (addr-of obj)))
 ||#
 
 (defun print-demodulator (demod &optional (stream *standard-output*) &rest ignore)
   (declare (ignore ignore))
   (let* ((lhs (demod-lhs demod))
-	 (rhs (demod-rhs demod))
-	 (clause (demod-clause demod))
-	 (clause-id (if (not (clause-p clause))
-			:*
-		      (clause-id clause)))
-	 (.printed-vars-so-far. nil))
+         (rhs (demod-rhs demod))
+         (clause (demod-clause demod))
+         (clause-id (if (not (clause-p clause))
+                        :*
+                      (clause-id clause)))
+         (.printed-vars-so-far. nil))
     (let ((*standard-output* stream)
-	  (.file-col. .file-col.)
-	  (indent 0))
+          (.file-col. .file-col.)
+          (indent 0))
       (format t "(~a) " clause-id)
       (setq indent (file-column stream))
       (setq .printed-vars-so-far.
-	(term-print lhs))
+        (term-print lhs))
       (setq .file-col. (file-column stream))
       (print-check indent .file-col.)
       (princ " --> ")
