@@ -1,6 +1,6 @@
 ;;;-*- Mode:LISP; Package:CHAOS; Base:10; Syntax:Common-lisp -*-
 ;;;
-;;; Copyright (c) 2000-2014, Toshimi Sawada. All rights reserved.
+;;; Copyright (c) 2000-2015, Toshimi Sawada. All rights reserved.
 ;;;
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -190,8 +190,10 @@ NOTE: this switch is obsolete now. please use `print mode' switch instead."
        "set path of user defined \"BOOL\" module."
        chaos-set-bool-path)
       ;; debug flags : invisible from user, internal use only
+      ("no" ("idcomp") parity *no-id-completion* "" nil nil t)
       ("sys" ("universal-sort") parity *allow-universal-sort* "" nil nil t)
       ("debug" ("rewrite") parity *rewrite-debug* "" nil nil t)
+      ("debug" ("memo") parity *memo-debug* "" nil nil t)
       ("debug" ("hash") parity *on-term-hash-debug* "" nil nil t)
       ("debug" ("axiom") parity *on-axiom-debug* "" nil nil t)
       ("debug" ("beh") parity *beh-debug* "" nil nil t)
@@ -214,8 +216,7 @@ NOTE: this switch is obsolete now. please use `print mode' switch instead."
       ("debug" ("apply") parity *apply-debug* "" nil nil t)
       ("debug" ("meta") parity *debug-meta* "" nil nil t)
       ("debug" ("citp") parity *debug-citp* "" nil nil t)
-      ;;
-      ("m-and-also" () general nil "" set-m-and-also nil t)
+      ("debug" ("print") parity *debug-print* "" nil nil t)
       ))
 
 (defun set-chaos-switch (which value)
@@ -258,9 +259,9 @@ NOTE: this switch is obsolete now. please use `print mode' switch instead."
       t
     (if (or (equal "off" x) (equal "no" x))
         nil
-	  (progn
-	    (princ "Specify on(yes) or off(no).") (terpri)
-	    (throw 'parity-error nil)))))
+          (progn
+            (princ "Specify on(yes) or off(no).") (terpri)
+            (throw 'parity-error nil)))))
 
 
 (defun show-modes (flg)
@@ -273,11 +274,11 @@ NOTE: this switch is obsolete now. please use `print mode' switch instead."
     (let ((sw flg))
       (if (or (equal sw '(".")) (null sw))
           (show-modes t)
-	    (let ((which (car flg))
+            (let ((which (car flg))
               (sub (cdr flg))
               (found nil)
               (cand nil))
-	      (dolist (sw *chaos-switches*)
+              (dolist (sw *chaos-switches*)
             (block next
               (let ((key (chaos-switch-key sw)))
                 (when (eq key :comment) (return-from next nil))
@@ -289,11 +290,11 @@ NOTE: this switch is obsolete now. please use `print mode' switch instead."
                       (setq found sw)
                       (setq cand nil)
                       (return)))))))
-	      (unless (or found cand)
+              (unless (or found cand)
             (with-output-chaos-warning ()
               (format t "unknown switch ~a" flg)
               (return-from show-modes nil)))
-	      (if found
+              (if found
               (show-mode found)
             (show-mode cand)))))))
 
@@ -304,10 +305,12 @@ NOTE: this switch is obsolete now. please use `print mode' switch instead."
         (type (chaos-switch-type switch)))
     (cond ((eq name :comment)
            (format t "~%~a" (second switch)))
+          ((equal name "libpath")
+           (format t "~%libpath~24T= ~{~a~^:~}" value))
           (t (when (atom name) (setq name (list name)))
              (if (eq type 'parity)
-                 (format t "~&~{~a~^|~a~} ~{~^ ~a~} ~24T~:[off~;on~]" name option value)
-               (progn (format t "~&~{~a~^|~a~} ~{~^ ~a~} ~24T= " name option)
+                 (format t "~%~{~a~^|~a~} ~{~^ ~a~} ~24T~:[off~;on~]" name option value)
+               (progn (format t "~%~{~a~^|~a~} ~{~^ ~a~} ~24T= " name option)
                       (if value
                           (print-chaos-object value)
                         (princ "not specified"))))))))
@@ -321,11 +324,11 @@ NOTE: this switch is obsolete now. please use `print mode' switch instead."
               (t (when (atom key) (setq key (list key)))
                  (case (chaos-switch-type sw)
                    (parity
-                    (format t "~& set ~{~a~^|~a~}~{~^ ~a~} {on|off} : ~a"
+                    (format t "~% set ~{~a~^|~a~}~{~^ ~a~} {on|off} : ~a"
                             key
                             (chaos-switch-subkey sw)
                             (chaos-switch-doc sw)))
-                   (otherwise (format t "~& set ~{~a~^|~a~}~{~^ ~a~} <value> : ~a"
+                   (otherwise (format t "~% set ~{~a~^|~a~}~{~^ ~a~} <value> : ~a"
                                       key
                                       (chaos-switch-subkey sw)
                                       (chaos-switch-doc sw))))))))))
@@ -345,13 +348,19 @@ NOTE: this switch is obsolete now. please use `print mode' switch instead."
 ;;; some switch setters
 ;;;
 (defun chaos-set-search-path (path)
-  (let* ((add (if (equal "+" (car path))
-                  t
-                nil))
-         (paths (if add (cadr path) (car path))))
+  (let* ((add (equal "+" (car path)))
+         (minus (equal "-" (car path)))
+         (paths (if (or add minus) (cadr path) (car path))))
+    (unless paths
+      (with-output-chaos-warning ()
+        (format t "No pathnames are specified.")
+        (return-from chaos-set-search-path nil)))
     (if add
         (set-search-path-plus paths)
-      (set-search-path paths))))
+      (if minus
+          (set-search-path-minus paths)
+          (set-search-path paths)))
+    (pr-search-path)))
 
 (defun chaos-set-tram-path (path)
   (let ((path (car path)))
@@ -384,10 +393,10 @@ NOTE: this switch is obsolete now. please use `print mode' switch instead."
         (parse-integer (car value) :junk-allowed t)
       (if (= len (length (car value)))
           (setq *cexec-limit* num)
-	    (with-output-chaos-error ('invalid-value)
-	      (format t "invalid value for exec limit: ~a" (car value))
-	      (print-next)
-	      (princ "must be a positive integer.") )))))
+            (with-output-chaos-error ('invalid-value)
+              (format t "invalid value for exec limit: ~a" (car value))
+              (print-next)
+              (princ "must be a positive integer.") )))))
 
 (defun chaos-set-print-depth (value)
   (if (or (null value)
@@ -397,10 +406,10 @@ NOTE: this switch is obsolete now. please use `print mode' switch instead."
         (parse-integer (car value) :junk-allowed t)
       (if (= len (length (car value)))
           (setq *term-print-depth* num)
-	    (with-output-chaos-error ('invalid-value)
-	      (format t "invalid value for term print depth: ~a" (car value))
-	      (print-next)
-	      (princ "must be a positive integer."))))))
+            (with-output-chaos-error ('invalid-value)
+              (format t "invalid value for term print depth: ~a" (car value))
+              (print-next)
+              (princ "must be a positive integer."))))))
 
 (defun chaos-set-print-mode (value)
   (case-equal (car value)
@@ -438,14 +447,6 @@ NOTE: this switch is obsolete now. please use `print mode' switch instead."
       (setq path (car path)))
     (setq *user-bool* path)))
 
-;;;
-(defun set-m-and-also (value)
-  (cond ((string= (car value) "on")
-	 (setq *m-and-op* *bool-and-also*)
-	 (setq *m-or-op* *bool-or-else*))
-	(t
-	 (setq *m-and-op* *bool-and*)
-	 (setq *m-or-op* *bool-or-else*))))
 
 ;;; EOF
 
