@@ -1,6 +1,6 @@
 ;;;-*- Mode:Lisp; Syntax:Common-Lisp; Package:CHAOS -*-
 ;;;
-;;; Copyright (c) 2000-2014, Toshimi Sawada. All rights reserved.
+;;; Copyright (c) 2000-2015, Toshimi Sawada. All rights reserved.
 ;;;
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -52,8 +52,8 @@
            (values t))
   (when (and (eql #\~ (char fname 0)) (eql #\/ (char fname 1)))
     (setq fname
-	  (concatenate 'string
-        (namestring (user-homedir-pathname))
+          (concatenate 'string
+            (namestring (user-homedir-pathname))
         (subseq fname 2))))
   (load fname))
 
@@ -102,18 +102,16 @@
     (probe-file (concatenate 'string dpath "/"))
     #+:SBCL
     (let ((directory-delimiter "/")  ; sbcl uses / on all platforms!
-	  (p (probe-file dpath)))
-      ; (format t "DEBUG is-directory? dpath ~s path ~s p ~s~%" dpath path p)
+          (p (probe-file dpath)))
       (if p
-	  (and (string-equal (subseq (namestring p)
-				     (1- (length (namestring p))))
-			     directory-delimiter)
-	       p)
-	nil))
+          (and (string-equal (subseq (namestring p)
+                                     (1- (length (namestring p))))
+                             directory-delimiter)
+               p)
+        nil))
     #+:Allegro
     (if (excl:file-directory-p dpath)
-        ;; (concatenate 'string dpath "/")
-        (pathname dpath)
+        (pathname (concatenate 'string dpath "/"))
       nil)
     #+(and :CCL (not :openmcl)) (if (directoryp dpath) dpath nil)
     #+:CLISP
@@ -135,10 +133,29 @@
             nil))
       (error "is-simple-file-name? : given non string arg ~a" path))))
 
+(defun is-relative-file-name? (path)
+  (declare (type (or simple-string pathname) path)
+           (values (or null t)))
+  (let ((pn (if (pathnamep path)
+		path
+	      (if (stringp path)
+		  (pathname path)
+		(error "is-relative-file-name? : given non string arg ~a" path)))))
+    (if (or (null (pathname-directory pn))
+	   (member ':relative (pathname-directory pn) :test #'equal))
+	t
+      nil)))
+
 (defun supply-suffixes (path suffixes)
   (declare (type (or simple-string pathname) path)
            (type list suffixes)
            (values list))
+  #+:SBCL
+  (when (position #\. (if (pathnamep path)
+                          (pathname-name path)
+                        path) 
+                  :from-end t)
+    (return-from supply-suffixes (list path)))
   (mapcar #'pathname
           (mapcar #'(lambda (x) (concatenate 'string
                                   (namestring path)
@@ -149,55 +166,56 @@
   (declare (type (or simple-string pathname) file)
            (type (or simple-string list) load-path)
            (type list suffixes))
-  (when (pathnamep file)
+  (when (and (pathnamep file) (not (is-directory? file)))
     (return-from chaos-probe-file (probe-file file)))
   ;;
   (setq file (expand-file-name file))
   (when (atom suffixes)
     (setq suffixes (list suffixes)))
-  (if load-path
-      (when (atom load-path)
-        (setq load-path (list load-path))))
+  (when (and load-path (atom load-path))
+    (setq load-path (list load-path)))
   ;;
-  (cond ((is-simple-file-name? file)
+  (cond ((is-relative-file-name? file)
          (let ((file-path (chaos-get-relative-path*
                            (concatenate 'string "./" file)))
                (res nil))
            ;;
            (setq res (probe-file file-path))
+           (when (and res (is-directory? res))
+             (setq res nil))
            (unless res
              (dolist (fx (supply-suffixes file-path suffixes))
-               (when (probe-file fx)
+               (when (and (probe-file fx) (not (is-directory? fx)))
                  (setq res fx)
                  (return)))
              ;; search through load paths
              (unless res
                (dolist (lpath load-path)
                  (let ((libdir (is-directory? lpath)))
-		   (declare (type (or null string pathname) libdir))
+                   (declare (type (or null string pathname) libdir))
                    (when libdir
-		     (unless (pathnamep libdir) (setq libdir (pathname libdir)))
+                     (unless (pathnamep libdir) (setq libdir (pathname libdir)))
                      (let ((f (make-pathname
-			       :host (pathname-host libdir)
-			       :device (pathname-device libdir)
+                               :host (pathname-host libdir)
+                               :device (pathname-device libdir)
                                :directory
-			       #+:CLISP libdir
-			       ;; #+:Allegro (namestring libdir)
-			       #-:CLISP (pathname-directory libdir)
+                               #+:CLISP libdir
+                               ;; #+:Allegro (namestring libdir)
+                               #-:CLISP (pathname-directory libdir)
                                :name (namestring file))))
-                       (if (probe-file f)
+                       (if (and (probe-file f) (not (is-directory? f)))
                            (progn (setq res f) (return))
                          (let ((x (supply-suffixes f suffixes)))
                            (dolist (fx x)
-                             (when (probe-file fx)
+                             (when (and (probe-file fx) (not (is-directory? fx)))
                                (setq res fx)
                                (return)))))))))))
            res))
         (t (let ((file-path (chaos-get-relative-path* file)))
-             (if (probe-file file-path)
+             (if (and (probe-file file-path) (not (is-directory? file-path)))
                  file-path
                (dolist (fx (supply-suffixes file-path suffixes))
-                 (when (probe-file fx)
+                 (when (and (probe-file fx) (not (is-directory? fx)))
                    (return-from chaos-probe-file fx))))))))
 
 (defun bare-chaos-pwd ()
@@ -215,9 +233,9 @@
 (defun chaos-relative-pathname? (f-name)
   (let ((fdp (pathname-directory (pathname f-name))))
     (or (null fdp)
-	(and fdp			; not simple file name.
-	     (not (eq (car (pathname-directory (pathname f-name)))
-		      :root))))))
+        (and fdp                        ; not simple file name.
+             (not (eq (car (pathname-directory (pathname f-name)))
+                      :root))))))
 
 (defun chaos-get-relative-path (f-name)
   (setq f-name (expand-file-name f-name))
@@ -226,11 +244,11 @@
 ; #+:SBCL
 ; (defun chaos-get-directory (file-path)
 ;   (let* ((ns (namestring file-path))
-; 	 (dpos (position #\/ ns :from-end t))
-; 	 (dir nil))
+;        (dpos (position #\/ ns :from-end t))
+;        (dir nil))
 ;     (unless dpos
 ;       (with-output-chaos-error ('internal-error)
-; 	(format t ":get-relative-path: could not find proper directory path, ~a" file-path)))
+;       (format t ":get-relative-path: could not find proper directory path, ~a" file-path)))
 ;     (subseq ns 0 (1+ dpos))))
 
 #+(or :Allegro :SBCL)
@@ -238,8 +256,8 @@
   (unless (pathnamep file-path)
     (setq file-path (pathname file-path)))
   (let ((dir-path (make-pathname :host (pathname-host file-path)
-				 :device (pathname-device file-path)
-				 :directory (pathname-directory file-path))))
+                                 :device (pathname-device file-path)
+                                 :directory (pathname-directory file-path))))
     ;;(namestring dir-path)
     dir-path))
 
@@ -250,25 +268,25 @@
         (let ((f-path nil))
           (unwind-protect
               (let ((host (pathname-host (pathname f-name)))
-		    (device (pathname-device (pathname f-name)))
-		    (fd (pathname-directory (pathname f-name)))
+                    (device (pathname-device (pathname f-name)))
+                    (fd (pathname-directory (pathname f-name)))
                     (f (file-namestring (pathname f-name))))
                 ;; #-GCL (declare (ignore fd))
-		;; (chaos-pushd (directory-namestring *chaos-input-source*))
-		(chaos-pushd (chaos-get-directory *chaos-input-source*))
+                ;; (chaos-pushd (directory-namestring *chaos-input-source*))
+                (chaos-pushd (chaos-get-directory *chaos-input-source*))
                 #+GCL
                 (setq f-path (truename (make-pathname :directory fd :name f)))
                 #+:CLISP
                 (setq f-path (make-pathname
-			      :host host
-			      :device device
+                              :host host
+                              :device device
                               :directory fd ;; (pathname fd)
                               :name f))
                 #-(or GCL :CLISP)
                 (progn
                   (setq f-path (make-pathname
-				:host host
-				:device device
+                                :host host
+                                :device device
                                 :directory fd
                                 :name f))))
             (chaos-popd))
@@ -293,10 +311,10 @@
     #+GCL (system comm)
     #+EXCL (excl:shell comm)
     #+SBCL (apply #'sb-ext:run-program
-		  #+win32 "CMD" #-win32 "/bin/sh"
-		  #+win32 (list "/c" "dir") #-win32 (list  "-c" comm)
-		  :input nil :output *terminal-io*
-		  #+win32 '(:search t) #-win32 nil)
+                  #+win32 "CMD" #-win32 "/bin/sh"
+                  #+win32 (list "/c" "dir") #-win32 (list  "-c" comm)
+                  :input nil :output *terminal-io*
+                  #+win32 '(:search t) #-win32 nil)
     #+LUCID (lucid::%execute-system-command comm)
     #+CLISP (ext::shell comm)))
 
@@ -318,21 +336,21 @@
 (defvar *chaos-directory-stack* nil)
 
 (defun chaos-print-directory-stack (&optional (stream *standard-output*))
-  (format stream "~&~a" *chaos-directory-stack*))
+  (format stream "~%~a" *chaos-directory-stack*))
 
 (defun fsys-parse-number (tok)
   (declare (type (or simple-string pathname) tok))
   (if (stringp tok)
       (let ((minusp nil))
-	(if (char= (char tok 0) #\-)
-	    (setq minusp t)
-	  (unless (char= (char tok 0) #\+)
-	    (return-from fsys-parse-number
-	      (values tok nil))))
-	(let ((num (read-from-string tok)))
-	  (if (numberp num)
-	      (values num minusp)
-	    (values tok nil))))
+        (if (char= (char tok 0) #\-)
+            (setq minusp t)
+          (unless (char= (char tok 0) #\+)
+            (return-from fsys-parse-number
+              (values tok nil))))
+        (let ((num (read-from-string tok)))
+          (if (numberp num)
+              (values num minusp)
+            (values tok nil))))
     (values tok nil)))
 
 (defun chaos-pushd (arg &optional (always-return nil))
@@ -355,7 +373,11 @@
                (progn
                  (pop *chaos-directory-stack*)
                  nil))))
-          (t (chaos-pushd "+1")))))
+          (t (if (<= (length *chaos-directory-stack*) 1)
+                 (with-output-chaos-warning ()
+                   (format t "No other directory.")
+                   (return-from chaos-pushd nil))
+               (chaos-pushd "+1"))))))
 
 (defun chaos-popd (&optional num)
   (declare (ignore num))
@@ -391,9 +413,9 @@
                  (setq ng t))
     #+SBCL
     (if (setq directory-path (is-directory? path))
-	(progn
-	  (setq *default-pathname-defaults* directory-path)
-	  (sb-posix:chdir directory-path))
+        (progn
+          (setq *default-pathname-defaults* directory-path)
+          (sb-posix:chdir directory-path))
       (setq ng t))
     #+(and :CCL (not :openmcl))
     (if (setq directory-path (is-directory? path))
@@ -489,7 +511,22 @@
     (dolist (p (parse-with-delimiter paths #\:))
       (push p path))
     (setq *chaos-libpath*
-	  (append (nreverse path) *chaos-libpath*))))
+          (append (nreverse path) *chaos-libpath*))))
+
+(defun set-search-path-minus (paths)
+  (when (consp paths) (setq paths (car paths)))
+  (let ((path nil))
+    (dolist (p (parse-with-delimiter paths #\:))
+      (push p path))
+    (dolist (p path)
+      (if (not (member p *chaos-libpath* :test #'equal))
+          (with-output-chaos-warning ()
+            (format t "The path ~s does not in 'libpath'." p))
+        (setq *chaos-libpath* (remove p *chaos-libpath* :test #'equal))))
+    *chaos-libpath*))
+
+(defun pr-search-path (&optional (stream *standard-output*))
+  (format stream "libpath = ~{~a~^:~}" *chaos-libpath*))
 
 ;;;
 ;;; INITIALIZATION
